/*******************************************************************************
 * Copyright 2013 Danny Kunz
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.omnaest.cluster;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
public class ClusterConfiguration implements Serializable
{
  private static final long serialVersionUID = 3467444603535051856L;
  @XmlElementWrapper(name = "servers")
  @XmlElementRef
  private List<Server>      serverList       = new ArrayList<Server>();
  
  public List<Server> getServerList()
  {
    return this.serverList;
  }
  
  public ClusterConfiguration setServerList( List<Server> serverList )
  {
    this.serverList.clear();
    this.serverList.addAll( serverList );
    return this;
  }
  
  @Override
  public String toString()
  {
    StringBuilder builder = new StringBuilder();
    builder.append( "ClusterConfiguration [serverList=" );
    builder.append( this.serverList );
    builder.append( "]" );
    return builder.toString();
  }
  
  @Override
  public int hashCode()
  {
    final int prime = 31;
    int result = 1;
    result = prime * result + ( ( this.serverList == null ) ? 0 : this.serverList.hashCode() );
    return result;
  }
  
  @Override
  public boolean equals( Object obj )
  {
    if ( this == obj )
    {
      return true;
    }
    if ( obj == null )
    {
      return false;
    }
    if ( !( obj instanceof ClusterConfiguration ) )
    {
      return false;
    }
    ClusterConfiguration other = (ClusterConfiguration) obj;
    if ( this.serverList == null )
    {
      if ( other.serverList != null )
      {
        return false;
      }
    }
    else if ( !this.serverList.equals( other.serverList ) )
    {
      return false;
    }
    return true;
  }
  
}
